//
// Alien.cpp
//

//
// Contains the definition of the Alien class
//

#include "Alien.h"
#include "Hero.h"
#include "WorldManager.h"
#include "ResourceManager.h"
#include "EventStep.h"
#include "EventOut.h"
#include "EventNuke.h"
#include "LogManager.h"
#include "Explosion.h"

Alien::Alien() {
  // register for step and out events
  WorldManager &gw = WorldManager::getInstance();
  gw.registerInterest(this, STEP_EVENT);
  gw.registerInterest(this, NUKE_EVENT);
  
  // setup "alien" sprite
  ResourceManager &rm = ResourceManager::getInstance();
  Sprite *p_temp_sprite = rm.getSprite("alien");
  setSprite(p_temp_sprite, true);
  setSpriteSlowdown(4);

  // set game object properties
  setType("Alien");
  move_slowdown = 2;
  move_countdown = move_slowdown;
}

Alien::~Alien() {
  WorldManager &gw = WorldManager::getInstance();
  gw.unregisterInterest(this, STEP_EVENT);
  gw.unregisterInterest(this, NUKE_EVENT);
  // create an explosion
  Explosion *p_explosion = new Explosion;
  p_explosion -> setPos(this -> getPos());
}

int Alien::eventHandler(Event *e) {
  if( e->getType() == STEP_EVENT)
    {
      move();
      return 1;
    }

  if( e->getType() == OUT_EVENT)
    {
      out();
      return 1;
    }

  if( e->getType() == COLLISION_EVENT)
    {
      EventCollision *p_collision_event = static_cast <EventCollision *> (e);
      hit(p_collision_event);
      return 1;
    }

  if( e->getType() == NUKE_EVENT)
    {
      WorldManager &gw = WorldManager::getInstance();
      gw.markForDelete(this);
      return 1;
    }

  // ignore this event
  return 0;
}

// move the Alien
void Alien::move() {
  move_countdown--;
  if(move_countdown > 0)
    return;
  move_countdown = move_slowdown;

  Position new_pos(pos.getX() - 1, pos.getY());
  WorldManager &gw = WorldManager::getInstance();
  gw.moveObj(this, new_pos);

  return;
}

// delete Aliens who are out of bounds
void Alien::out() {
  if(getPos().getX() >= 0)
    {
      return;
    }

  WorldManager &gw = WorldManager::getInstance();
  gw.markForDelete(this);
}

// if the Alien hits the Hero, then handle the collision
void Alien::hit(EventCollision *p_c) {
  LogManager &lm = LogManager::getInstance();
  WorldManager &gw = WorldManager::getInstance();

  if((p_c -> getObj1() -> getType()) == "Hero")
    {
      lm.writeLog("Alien::hit(): Hit the Hero");
      gw.markForDelete(this);
      Hero *p_temp_hero = (Hero*)(p_c -> getObj1());
      // must be used instead of markForDelete(), because of Lives
      //   the Hero might not die.
      p_temp_hero->hit();
    }

  if((p_c ->getObj2() -> getType()) == "Hero")
    {
      lm.writeLog("Alien::hit(): Hit the Hero");
      gw.markForDelete(this);
      Hero *p_temp_hero = (Hero*)(p_c -> getObj2());
      p_temp_hero->hit();
    }

}
