//
// Hero.cpp
//

#include "Hero.h"
#include "Bullet.h"
#include "Mine.h"
#include "Lives.h"
#include "GameOver.h"
#include "Explosion.h"
#include "LogManager.h"
#include "InputManager.h"
#include "ResourceManager.h"
#include "WorldManager.h"
#include "EventStep.h"
#include "EventDeath.h"
#include "GraphicsManager.h"
#include "GameManager.h"

Hero::Hero() {

  // link to "ship" sprite
  ResourceManager &rm = ResourceManager::getInstance();
  LogManager &lm = LogManager::getInstance();
  Sprite *p_temp_sprite;
  p_temp_sprite = rm.getSprite("ship");
  if (!p_temp_sprite) {
    lm.writeLog("Hero::Hero(): Warning! Sprite '%s' not found", "ship");
    return;
  }
  setSprite(p_temp_sprite, true); // center and define box
  setSpriteSlowdown(3);		// third speed animation

  // player controls hero, so register with keyboard
  InputManager &im = InputManager::getInstance();
  im.registerInterest(this, KEYBOARD_EVENT);

  // need to update fire rate control each step
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.registerInterest(this, STEP_EVENT);
  world_manager.registerInterest(this, DEATH_EVENT);

  setType("Hero");

  // set starting location
  pos.setX(7);
  pos.setY(world_manager.getBoundary().getVertical()/2);

  fire_slowdown = 15;
  fire_countdown = fire_slowdown;

  nuke_count = 1;
  mine_count = 10;

  lives = 3;
  for(int i = 0; i < lives; i++)
    {
      Lives *p_l = new Lives(i);
    }
}

Hero::~Hero() {
  InputManager &im = InputManager::getInstance();
  im.unregisterInterest(this, KEYBOARD_EVENT);
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.unregisterInterest(this, STEP_EVENT);
  world_manager.unregisterInterest(this, DEATH_EVENT);

  string temp = "gameover";
  GameOver *p_go = new GameOver(temp);

  for(int i = 0; i<lives; i++)
    {
      EventDeath death;
      world_manager.onEvent(&death);
    }
  
  // make big explosion
  for (int i = -8; i <= 8; i+= 5) {
    for (int j=-5; j <= 5; j+= 3) {
      Position temp_pos = this->getPos();
      temp_pos.setX(this->getPos().getX() + i);
      temp_pos.setY(this->getPos().getY() + j);
      Explosion *p_explosion = new Explosion;
      p_explosion -> setPos(temp_pos);
    }
  }
}

// handle event
// return 0 if ignored, else 1
int Hero::eventHandler(Event *e) {

  if (e->getType() == KEYBOARD_EVENT) {
    EventKeyboard *p_keyboard_event = static_cast <EventKeyboard *> (e);
    kbd(p_keyboard_event);
    return 1;
  }

  if (e->getType() == STEP_EVENT) {
    step();
    return 1;
  }

  if (e->getType() == DEATH_EVENT) {
    dead();
    return 1;
  }

  // if we get here, we have ignored this event
  return 0;
}

// call move (or do nothing) according to key pressed
void Hero::kbd(EventKeyboard *p_keyboard_event) {

  switch(p_keyboard_event->getKey()) {
  case KEY_UP:			// up arrow
    move(-1);
    break;
  case KEY_DOWN:		// down arrow
    move(+1);
    break;
  case 'm':
    dropMine();
    break;
  case ' ':			// fire
    fire();
    break;
  case 13:
    nuke();
    break;
  };
  return;
}

// move up or down
void Hero::move(int dy) {
  WorldManager &world_manager = WorldManager::getInstance();
  Position new_pos(pos.getX(), pos.getY() + dy);

  // if stays on screen, allow move
  if ((new_pos.getY() > 0) && 
      (new_pos.getY() < world_manager.getBoundary().getVertical()))
    world_manager.moveObj(this, new_pos);
}

// fire a bullet
void Hero::fire() {
  if (fire_countdown > 0)
    return;
  fire_countdown = fire_slowdown;
  new Bullet(pos);
}

// decrease fire restriction
void Hero::step() {
  fire_countdown--;
  if (fire_countdown < 0)
    fire_countdown = 0;
}

// launch the nuke
void Hero::nuke() {
  // check if nukes left
  if (!nuke_count)
    return;
  nuke_count--;

  // create a "nuke" event and send to all interested objects
  WorldManager &world_manager = WorldManager::getInstance();
  EventNuke nuke;
  world_manager.onEvent(&nuke);
}

void Hero::hit() {
  LogManager &lm = LogManager::getInstance();
  lm.writeLog("Hero killed");
  WorldManager &wm = WorldManager::getInstance();
  EventDeath death;
  wm.onEvent(&death);
}

void Hero::dead() {
  lives--;
  if(lives <= 0) {
    WorldManager &wm = WorldManager::getInstance();
    wm.markForDelete(this);
  }
}

void Hero::dropMine() {
  new Mine(pos);
}
