//
// Lives.cpp
//

//
// Contains the class declaration of the Lives class
//   The class represents a single cross in the game HUD
//

#include "Lives.h"
#include "EventDeath.h"
#include "Explosion.h"
#include "WorldManager.h"
#include "ResourceManager.h"
#include "LogManager.h"

Lives::Lives(int id) {
  // register for DEATH_EVENT
  WorldManager &wm = WorldManager::getInstance();
  wm.registerInterest(this, DEATH_EVENT);

  // set up sprite
  LogManager &lm = LogManager::getInstance();
  ResourceManager &rm = ResourceManager::getInstance();
  Sprite *p_temp_sprite = rm.getSprite("life");
  if(!p_temp_sprite)
    {
      lm.writeLog("Lives::Lives(): Warning! Couldn't find sprite '%s'.","life");
    }
  setSprite(p_temp_sprite, true);

  // set GameObject properties
  setType("Life");
  altitude = 1;
  is_solid = false;

  // life_countdown determines how many DeathEvents this object lives through
  life_countdown = id;

  pos.setX(wm.getBoundary().getHorizontal() - 4*life_countdown - 4);
  pos.setY(1);
}

Lives::~Lives() {
  WorldManager &wm = WorldManager::getInstance();
  wm.unregisterInterest(this, DEATH_EVENT);
}

int Lives::eventHandler(Event *e) {
  if( e->getType() == DEATH_EVENT) {
    // during a Death Event, count down the life counter
    life_countdown--;
    if(life_countdown < 0)
      {
	// if the life is gone, delete it
	WorldManager &wm = WorldManager::getInstance();
	wm.markForDelete(this);
      }
  }
}
