//
// Mine.cpp
//

//
// Contains the definition for the Mine class
//   

#include "Mine.h"
#include "Hero.h"
#include "Explosion.h"
#include "WorldManager.h"
#include "ResourceManager.h"
#include "EventStep.h"
#include "EventOut.h"

Mine::Mine(Position hero_pos) {
  // register for the step event
  WorldManager &gw = WorldManager::getInstance();
  gw.registerInterest(this, STEP_EVENT);
  gw.registerInterest(this, OUT_EVENT);

  // set up sprite
  ResourceManager &rm = ResourceManager::getInstance();
  Sprite *p_temp_sprite = rm.getSprite("mine");
  setSprite(p_temp_sprite, true);

  // set mine properties
  setType("Mine");
  
  // the mine doesn't collide until it arms
  is_solid = false;
  time_to_arm = 20;
  setSpriteSlowdown(10);

  move_slowdown = 20;
  move_countdown = move_slowdown;
  explosionSize = 3;

  pos.setX(hero_pos.getX());
  pos.setY(hero_pos.getY()+3);
}

Mine::~Mine() {
  WorldManager &gw = WorldManager::getInstance();
  gw.unregisterInterest(this, STEP_EVENT);
  gw.unregisterInterest(this, OUT_EVENT);

  // display explosion
  Explosion *p_e = new Explosion;
  p_e->setPos(pos);
}

int Mine::eventHandler(Event *e) {
  if(e->getType() == STEP_EVENT)
    {
      step();
      return 1;
    }

  if(e->getType() == COLLISION_EVENT)
    {
      EventCollision *p_collision_event = static_cast <EventCollision *> (e);
      hit(p_collision_event);
      return 1;
    }

  if(e->getType() == OUT_EVENT)
    {
      // if the Mine falls under the world, delete it
      WorldManager &gw = WorldManager::getInstance();
      gw.markForDelete(this);
      return 1;
    }

  return 0;
}

void Mine::hit(EventCollision *p_c) {
  if(time_to_arm > 0)
    {
      return;
    }

  WorldManager &gw = WorldManager::getInstance();

  // check for nearby objects
  GameObjectList gameObjs = gw.getAllObjects();
  GameObjectListIterator go_iter = gameObjs.createIterator();

  while(!go_iter.isDone())
    {
      // if the object is within 3 units of the explosion, it is hit
      Position temp_pos = go_iter.currentObj() -> getPos();
      if((temp_pos.getX() < pos.getX()+explosionSize && 
	  temp_pos.getX() > pos.getX()-explosionSize) &&
	 (temp_pos.getY() < pos.getY()+explosionSize &&
	  temp_pos.getY() > pos.getY()+explosionSize))
	{
	  // object is caught in blast
	  if(go_iter.currentObj() -> getType() == "Hero")
	    {
	      // Hero's aren't automatically destroyed, it just loses a life
	      Hero *p_hero = (Hero*)go_iter.currentObj();
	      p_hero->hit();
	    }
	  else if(go_iter.currentObj() -> getType() == "Saucer" ||
		  go_iter.currentObj() -> getType() == "Alien" ||
		  go_iter.currentObj() -> getType() == "Mine")
	    {
	      gw.markForDelete(go_iter.currentObj());
	    }
	}

      go_iter.next();
    }
  gw.markForDelete(this);
}

void Mine::step() {
  time_to_arm--;
  if(time_to_arm <= 0)
    {
      // if the mine is armed, then it can collide
      is_solid = true;
      setSpriteSlowdown(1);
      
      // move the mine
      move_countdown--;
      if(move_countdown > 0)
	return;
      move_countdown = move_slowdown;

      Position new_pos(pos.getX(), pos.getY()+1);
      WorldManager &gw = WorldManager::getInstance();
      gw.moveObj(this, new_pos);      
    }
  else
    {
      return;
    }
}
