//
// TextMessage.cpp
//

#include "TextMessage.h"
#include "WorldManager.h"
#include "ResourceManager.h"
#include "LogManager.h"
#include "EventStep.h"

TextMessage::TextMessage(string message) {
  
  // link to "message" sprite
  ResourceManager &resource_manager = ResourceManager::getInstance();
  LogManager &lm = LogManager::getInstance();
  Sprite *p_temp_sprite = resource_manager.getSprite(message);
  if(!p_temp_sprite) {
    lm.writeLog("GameStart::GameStart(): Warning! Sprite '%s' not found",
		message.c_str());
    return;
  }
  setSprite(p_temp_sprite);

  is_solid = false;
  setSpriteSlowdown(15);
  time_to_live = p_temp_sprite->getFrameCount() * 15;

  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.registerInterest(this, STEP_EVENT);

  altitude = 2;         // make in the foreground

  // put in center of screen
  pos.setX(world_manager.getBoundary().getCorner().getX() +
	   world_manager.getBoundary().getHorizontal()/2);
  pos.setY(world_manager.getBoundary().getCorner().getY() +
	   world_manager.getBoundary().getVertical()/2);
}

TextMessage::~TextMessage() {
  WorldManager &world_manager = WorldManager::getInstance();
  world_manager.unregisterInterest(this, STEP_EVENT);
}

int TextMessage::eventHandler(Event *e) {
  if( e->getType() == STEP_EVENT) {
    step();
    return 1;
  }

  return 0;
}

void TextMessage::step() {
  time_to_live--;
  if (time_to_live <= 0) {
    WorldManager &wm = WorldManager::getInstance();
    wm.markForDelete(this);
  }
}
